/**
 * OW2 Util
 * Copyright (C) 2010-2011 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ProcessSession.java 5691 2011-01-05 22:47:22Z loris $
 * --------------------------------------------------------------------------
 */
package org.ow2.util.protocol.impl;

import java.io.ByteArrayOutputStream;
import java.io.IOException;

import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.ow2.util.protocol.api.IProtocolSession;
import org.ow2.util.protocol.api.ProtocolConfig;
import org.ow2.util.protocol.api.ProtocolException;

/**
 *
 * @author Loris Bouzonnet
 */
public class ProcessSession implements IProtocolSession {

    private static Log logger = LogFactory.getLog(ProcessSession.class);

    private final ProtocolConfig config;

    public ProcessSession(final ProtocolConfig config) {
        this.config = config;
    }

    public void close() {
    }

    public void send(final String command) throws ProtocolException {
        final Process process;
        try {
            process = Runtime.getRuntime().exec(command);
        } catch (final IOException e) {
            throw new ProtocolException(
                    "Error when starting a process for the command: " + command, e);
        }

        final StreamReader errStreamReader =
                new StreamReader(process.getErrorStream(), this.config.getErrorStream());
        new Thread(errStreamReader).start();

        final StreamReader inStreamReader =
                new StreamReader(process.getInputStream(), this.config.getOutputStream());
        new Thread(inStreamReader).start();

        final int exitCode;
        try {
            exitCode = process.waitFor();
            if (exitCode == 0) {
                logger.debug("Exit code: " + exitCode);
            } else {
                logger.warn("Exit code: " + exitCode);
            }
        } catch (final InterruptedException e) {
            Thread.currentThread().interrupt();
        }

    }

    public static void main(final String[] arg) throws ProtocolException {

        final ProtocolConfig config = new ProtocolConfig();
        config.setErrorStream(System.err);
        final ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        config.setOutputStream(outputStream);

        final ProcessSession session = new ProcessSession(config);
        session.send(arg[0]);
        session.close();

        System.out.println(outputStream.toString());
    }

}
